;; Generated package description from cape.el  -*- mode: lisp-data; no-byte-compile: t -*-
(define-package "cape" "1.9.0.20250308.121335" "Completion At Point Extensions" '((emacs "28.1") (compat "30")) :commit "cc9cf45c651ad4784444cfdbdf238ac18dce39c9" :authors '(("Daniel Mendler" . "mail@daniel-mendler.de")) :maintainer '("Daniel Mendler" . "mail@daniel-mendler.de") :keywords '("abbrev" "convenience" "matching" "completion" "text") :url "https://github.com/minad/cape")
